import { shortcut, store } from '@core/core';
import { ConnectionModel } from './connection.model';
import { PAGE_INDEX } from '@constants/constant';
import { ConnectionList } from './list/list';
import { ConnectionJdbc } from './connection_jdbc/connection_jdbc';
import { ConnectionJndi } from './connection_jndi/connection_jndi';
import { ConnectionPlugin } from './connection_plugin/connection_plugin';
import { connectionType } from '@constants/env';
import { getAllDatabaseTypes, connectionCanEdit, getJdbcDatabaseType, getTextByDatabaseType } from '../../app.service';
import { ConnectionJDBC } from '../../crud/crud.typings';
import { Button, HTapeLayout, Label, ListView } from '@fui/core';

@shortcut()
@store(ConnectionModel)
export class Connection extends BI.Widget {
    static xtype = 'dec.dcm.connection';

    store: ConnectionModel['store'];
    model: ConnectionModel['model'];

    connectionTitleWidget: Label;
    connectionEditWidget: Button;
    listView: ListView;
    title: HTapeLayout;

    watch = {
        connectionSelectedOne: {
            immediate: true,
            handler: (v: Connection) => {
                BI.nextTick(() => {
                    const connectionName = v.connectionName;

                    connectionName
                        ? this.renderConnectionListView(connectionName)
                        : this.renderEmptyListView();
                });
            },
        },
    };

    beforeRender(cb: Function) {
        this.store.initConnectionLicInfo(cb);
    }

    render() {
        return {
            type: BI.HTapeLayout.xtype,
            hgap: 10,
            items: [
                {
                    el: {
                        type: BI.VTapeLayout.xtype,
                        cls: 'bi-border-right',
                        rgap: 10,
                        items: [
                            {
                                el: {
                                    type: BI.LeftRightVerticalAdaptLayout.xtype,
                                    cls: 'bi-border-bottom',
                                    items: {
                                        left: [
                                            {
                                                type: BI.Button.xtype,
                                                text: BI.i18nText('Dec-Dcm_Connection_New'),
                                                title: BI.i18nText('Dec-Dcm_Connection_New'),
                                                minWidth: 0,
                                                width: 98,
                                                handler: () => {
                                                    this.store.createNewConnection();
                                                },
                                            },
                                        ],
                                        right: [
                                            {
                                                type: 'dec.connection.driver.entry',
                                                invisible: !BI.Services.getService('dec.service.global').isAdmin(),
                                                from: '.dec-dcm',
                                                listeners: [
                                                    {
                                                        eventName: 'EVENT_CLOSE',
                                                        action: () => {
                                                            this.reset();
                                                        },
                                                    },
                                                ],
                                            },
                                        ],
                                    },
                                },
                                height: 40,
                            },
                            {
                                type: ConnectionList.xtype,
                                tgap: 10,
                            },
                        ],
                    },
                    width: 275,
                },
                {
                    type: BI.VTapeLayout.xtype,
                    items: [
                        {
                            el: {
                                type: BI.HTapeLayout.xtype,
                                ref: (_ref: HTapeLayout) => {
                                    this.title = _ref;
                                },
                                cls: 'bi-border-bottom',
                                items: [
                                    {
                                        type: BI.Label.xtype,
                                        textAlign: 'left',
                                        ref: (_ref: Label) => {
                                            this.connectionTitleWidget = _ref;
                                        },
                                    },
                                    {
                                        el: {
                                            type: BI.VerticalAdaptLayout.xtype,
                                            items: [{
                                                type: BI.Button.xtype,
                                                $value: 'connection-edit',
                                                invisible: true,
                                                text: BI.i18nText('Dec-Dcm_Edit'),
                                                ref: (_ref: Button) => {
                                                    this.connectionEditWidget = _ref;
                                                },
                                                handler: () => {
                                                    this.store.getConnectionStatus()
                                                        .then(() => {
                                                            const databaseType = this.model.connectionSelectedOne.connectionType;
                                                            const database = BI.find(getAllDatabaseTypes(), (_index, value) => value.databaseType === databaseType);
                                                            this.setMaintainPage();
                                                            this.store.setNoTestConnection(database.isHideConnection);
                                                        })
                                                        .catch(() => {
                                                        });
                                                },
                                            }],
                                        },
                                    },
                                ],
                            },
                            height: 40,
                        },
                        {
                            type: BI.ListView.xtype,
                            ref: (_ref: ListView) => {
                                this.listView = _ref;
                            },
                        },
                    ],
                },
            ],
        };
    }

    mounted() {
        this.store.setDatebaseTypeSelected('');
    }

    private renderItems() {
        switch (this.model.connectionSelectedOne.connectionType) {
            case connectionType.JDBC:
                return [{
                    type: ConnectionJdbc.xtype,
                }];
            case connectionType.JNDI:
                return [{
                    type: ConnectionJndi.xtype,
                }];
            default:
                return [{
                    type: ConnectionPlugin.xtype,
                }];
        }
    }

    private renderNoRegistered() {
        return [{
            type: BI.CenterAdaptLayout.xtype,
            height: 500,
            items: [
                {
                    type: BI.VTapeLayout.xtype,
                    width: 300,
                    height: 150,
                    items: [
                        {
                            el: {
                                type: BI.Layout.xtype,
                                cls: 'error-page-background',
                            },
                            height: 130,
                        },
                        {
                            type: BI.Label.xtype,
                            cls: 'bi-tips',
                            text: BI.i18nText('Dec-Dcm_Connection_Np_Registered'),
                        },
                    ],
                },
            ],
        }];
    }

    private hasRegistered() {
        const allDatabaseTypes = getAllDatabaseTypes();
        switch (this.model.connectionSelectedOne.connectionType) {
            case connectionType.JDBC:
                return true;
            case connectionType.JNDI:
                return true;
            default:
                return allDatabaseTypes.some(item => item.databaseType === this.model.connectionSelectedOne.connectionType);
        }
    }

    private getSelectConnectionType() {
        let databaseType = this.model.connectionSelectedOne.connectionType;
        if (databaseType === connectionType.JDBC) {
            const connectionJDBC = this.model.connectionSelectedOne.connectionData as ConnectionJDBC;
            databaseType = getJdbcDatabaseType(connectionJDBC.database, connectionJDBC.driver).databaseType;
        }

        return databaseType;
    }

    private setMaintainPage() {
        this.store.setPageIndex(PAGE_INDEX.MAINTAIN);
        this.store.setDatebaseTypeSelected('');
    }

    private renderConnectionListView(name: string) {
        const canEdit = connectionCanEdit(this.model.connectionSelectedOne),
            type = this.getSelectConnectionType(),
            hasRegistered = this.hasRegistered();

        this.connectionTitleWidget.setText(`${name}（${getTextByDatabaseType(type)}）`);
        this.connectionEditWidget.setVisible(canEdit);

        this.title.setVisible(hasRegistered);

        hasRegistered
            ? this.listView.populate(BI.createItems(this.renderItems()))
            : this.listView.populate(BI.createItems(this.renderNoRegistered()));
    }

    private renderEmptyListView() {
        this.listView.populate(
            BI.createItems([
                {
                    type: BI.Layout.xtype,
                }
            ])
        );

        this.connectionTitleWidget.setText('');
        this.connectionEditWidget.setVisible(false);
    }
}
